# == Schema Information
#
# Table name: media_objs
#
#  id                :integer          not null, primary key
#  file_file_name    :string(255)
#  file_content_type :string(255)
#  file_file_size    :integer
#  file_updated_at   :datetime
#  type              :string(255)
#  creator_type      :string(255)
#  creator_id        :integer
#  created_at        :datetime         not null
#  updated_at        :datetime         not null
#
# Indexes
#
#  index_media_objs_on_creator_type_and_creator_id  (creator_type,creator_id)
#

module Whaleback
  module Media
    class Obj < ApplicationRecord
      belongs_to :creator, polymorphic: true, optional: true
      has_many :attachments

      has_attached_file :file
      validates_attachment_presence :file

      before_destroy :stop_destroying, unless: :useless

      TypeTrans = {
        ''         => 'Whaleback::Media::Obj',
        'image'    => 'Whaleback::Media::Obj::Image',
        'document' => 'Whaleback::Media::Obj::Document',
        'video'    => 'Whaleback::Media::Obj::Video',
        'voice'    => 'Whaleback::Media::Obj::Voice',
      }

      class << self
        def media_klass media_type
          Media::Obj::TypeTrans[ media_type.to_s ].to_s.safe_constantize || raise_wrong_media_type(media_type)        
        end

        def raise_wrong_media_type
          raise Whaleback::Errors::BaseError.new("Invalid media_type: #{media_type}", -1, 400)          
        end
      end

      attr_accessor :default_mode
      
      def url mode=nil
        ActionController::Base.helpers.asset_url(
          mode || default_mode ? file.url(mode) : file
        )
      end

      def name
        file_file_name
      end

      def size
        file_file_size
      end

      def media_type
        TypeTrans.invert[type]
      end

      def media_type= val
        self.type = TypeTrans[val]
      end
      
      def useless
        attachments.empty?
      end

      def stop_destroying
        raise Whaleback::Errors::BaseError.new('This media obj being used.', -1, 400)
      end
      
      require_relative 'obj/document'
      require_relative 'obj/image'
      require_relative 'obj/video'
      require_relative 'obj/voice'
    end
  end
end
